import 'dart:convert';
import 'dart:io';

import 'package:crp_core/auth/provider/jwt/JwtToken.dart';
import 'package:crp_core/core/exception/AppException.dart';
import 'package:crp_core/core/http/AppHttpResponse.dart';
import 'package:crp_core/core/model/AuthToken.dart';
import 'package:crp_core/core/provider/AuthProvider.dart';
import 'package:crp_core/env/EnvConfig.dart';
import 'package:http/http.dart' as http;

class JwtProvider extends AuthProvider<JwtToken> {
  String _apiUrl;

  JwtProvider(this._apiUrl);

  @override
  Future<AuthToken> auth(Map<String, String> credentials, {Map<String, String> header}) async {
    try {
      var response = await http
          .post("${Env.instance.config.authApiUrl}$_apiUrl", body: credentials, headers: header);
      var body = returnResponse(
          AppHttpResponse(code: response.statusCode, body: response.body));
      return JwtToken.fromJson(jsonDecode(body));
    } on SocketException {
      throw FetchDataException('Sem conexão à Internet');
    }
  }
}
