import 'dart:io';

import 'package:crp_core/core/exception/AppException.dart';
import 'package:crp_core/core/http/AppHttpResponse.dart';
import 'package:crp_core/core/model/Entity.dart';
import 'package:crp_core/core/provider/Provider.dart';

typedef S ClientRequest<S>();

abstract class RestProvider<T extends Entity> extends Provider<T> {
  String _apiUrl;

  RestProvider(String apiUrl) {
    _apiUrl = apiUrl;
  }

  dynamic getClient();

  AppHttpResponse createHttpResponse(dynamic response);

  Future<dynamic> fetchAll() async {
    return await fetch(() => getClient().get(_apiUrl));
  }

  Future<dynamic> fetchById(dynamic id) async {
    return await fetch(() => getClient().get("$_apiUrl/$id"));
  }

  Future<dynamic> delete(dynamic id) async {
    return await fetch(() => getClient().delete("$_apiUrl/$id"));
  }

  Future<dynamic> update(T instance) async {
    return await fetch(() => getClient()
        .put("$_apiUrl/${instance.getId()}", data: instance.toJson()));
  }

  Future<dynamic> insert(T instance) async {
    return await fetch(
        () => getClient().post("$_apiUrl", data: instance.toJson()));
  }

  Future<dynamic> fetch(ClientRequest request) async {
    try {
      return returnResponse(createHttpResponse(await request()));
    } on SocketException {
      throw FetchDataException('Sem conexão à Internet');
    } catch (e) {
      return returnResponse(createHttpResponse(e));
    }
  }

  String get apiUrl => _apiUrl;
}
