import 'package:crp_core/core/model/Entity.dart';
import 'package:crp_core/core/provider/Provider.dart';

abstract class Repository<T extends Entity> {
  Provider<T> getApiProvider();

  T fromMap(Map<String, dynamic> json);

  Future<List<T>> fetchAll() async {
    final response = await getApiProvider().fetchAll();
    if (response != null) {
      return response.map<T>((json) => fromMap(json)).toList();
    } else
      return null;
  }

  Future<T> fetchById(int id) async {
    final response = await getApiProvider().fetchById(id);
    if (response != null) {
      return fromMap(response);
    } else
      return null;
  }

  Future<void> delete(int id) async {
    await getApiProvider().delete(id);
  }

  Future<void> update(T instance) async {
    await getApiProvider().update(instance);
  }

  Future<void> insert(T instance) async {
    await getApiProvider().insert(instance);
  }
}
