import 'dart:convert';

import 'package:crp_core/core/exception/AppException.dart';
import 'package:crp_core/core/model/AuthToken.dart';
import 'package:shared_preferences/shared_preferences.dart';

class AuthTokenStorage {
  static final AuthTokenStorage instance = AuthTokenStorage._();

  AuthTokenStorage._();

  Future<AuthToken> getAuthToken<T extends AuthToken>() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    String token = prefs.get("auth_token");
    if (token == null) {
      throw TokenStorageException("Token não encontrado");
    }
    return AuthToken.fromJson(jsonDecode(token));
  }

  Future<void> putAuthToken<T extends AuthToken>(T token) async {
    try {
      SharedPreferences prefs = await SharedPreferences.getInstance();
      prefs.setString("auth_token", jsonEncode(token));
    } catch (e) {
      throw TokenStorageException("Falha ao inserir o token nas preferências!");
    }
  }
}
