import 'package:crp_core/core/http/AppHttpResponse.dart';
import 'package:crp_core/core/model/Entity.dart';
import 'package:crp_core/core/provider/RestProvider.dart';
import 'package:crp_core/core/provider/interceptor/AuthTokenInterceptor.dart';
import 'package:crp_core/env/EnvConfig.dart';
import 'package:dio/dio.dart';

class DioRestProvider<T extends Entity> extends RestProvider<T> {
  Dio client = Dio();
  bool withTokenInterceptor = false;

  DioRestProvider(String apiUrl) : super(apiUrl) {
    client.interceptors.add(TokenInterceptor());
    client.interceptors.add(LogInterceptor());
    client.options.baseUrl = Env.instance.config.apiUrl;
    client.options.connectTimeout = 50000;
  }

  DioRestProvider.tokenInterceptor(String apiUrl) : super(apiUrl) {
    client.interceptors.add(TokenInterceptor());
    client.interceptors.add(LogInterceptor());
    client.options.baseUrl = Env.instance.config.apiUrl;
    client.options.connectTimeout = 50000;
  }

  @override
  AppHttpResponse createHttpResponse(requestResponse) {
    if (requestResponse is Response) {
      return makeFromDioResponse(requestResponse);
    } else if (requestResponse is DioError) {
      if (requestResponse.response != null) {
        return makeFromDioResponse(requestResponse.response);
      } else {
        return AppHttpResponse(code: 0, body: requestResponse.message);
      }
    }
    return AppHttpResponse(
        code: 0, body: "Não foi possivel identificar o error na requisição");
  }

  AppHttpResponse makeFromDioResponse(Response response) {
    return AppHttpResponse(code: response.statusCode, body: response.data);
  }

  getTokenClient() {
    client.options.extra["token"] = true;
    return client;
  }

  @override
  getClient() {
    client.options.extra["token"] = false;
    return client;
  }
}
